#include <iostream>
#include <unordered_set>
using namespace std;

// Represents a 2D coordinate
class Coord
{
    int x,y;
public:
    Coord(int x, int y) : x(x), y(y) {}
    friend bool operator==(const Coord& c1,const Coord& c2); // give 'operator=='         access to private members
    friend struct hash<Coord>;                               // give 'struct hash<Coord>' access to private members
    friend ostream& operator<<(ostream& os, const Coord& c); // give 'operator<<'         access to private members
};

// Test two Coords for equality
bool operator==(const Coord& c1,const Coord& c2) 
{ return c1.x==c2.x && c1.y==c2.y; }

// Define hash function for a Coord in std namespace
namespace std
{
    template <>
    struct hash<Coord>
    {
        // Compute the hash value of a Coord
        size_t operator()(const Coord& c) const 
        {
            return hash<int>()(c.x) ^ hash<int>()(c.y);
        }
    };
}

// Stream a Coord to ouput stream (for printing)
ostream& operator<<(ostream& os, const Coord& c) 
{   os << "(" << c.x << "," << c.y << ")"; return os; }

// Stream an unordered_set of any type to output stream (for printing)
template <typename T>
ostream& operator<<(ostream& os, const unordered_set<T>& c)
{
    for (const auto& x : c)
        os << x << ", ";
    return os;
}

int main()
{
    cout<<"--- initialize"<<endl;
    unordered_set<Coord> world { Coord(1,0), Coord(1,0), Coord(2,1), Coord(2,3) };
    cout<<"world: "<< world <<endl; // unordered_set has no duplicates
    cout<<"world.size(): "<< world.size() <<endl;

    cout<<endl<<"--- insert"<<endl;
    world.insert(Coord(3,1)); // add a Coord
    cout<<"world: "<< world <<endl;
    cout<<"world.size(): "<< world.size() <<endl;

    cout<<endl<<"--- remove"<<endl;
    world.erase(Coord(1,0)); // remove a Coord
    cout<<"world: "<< world <<endl;
    cout<<"world.size(): "<< world.size() <<endl;
    
    cout<<endl<<"--- search"<<endl;
    cout<<"world.count(Coord(2,1)): "<< world.count(Coord(2,1)) <<endl;
    cout<<"world.count(Coord(1,0)): "<< world.count(Coord(1,0)) <<endl;
}
